@@ -18,37 +18,40 @@ import (
 )
 
 // ensureDNSService ensures that a service exists for a given DNS.
-func (r *reconciler) ensureDNSService(dns *operatorv1.DNS, clusterIP string, daemonsetRef metav1.OwnerReference) (*corev1.Service, error) {
-	current, err := r.currentDNSService(dns)
+func (r *reconciler) ensureDNSService(dns *operatorv1.DNS, clusterIP string, daemonsetRef metav1.OwnerReference) (bool, *corev1.Service, error) {
+	haveService, current, err := r.currentDNSService(dns)
 	if err != nil {
-		return nil, err
+		return false, nil, err
 	}
 	desired := desiredDNSService(dns, clusterIP, daemonsetRef)
 
 	switch {
-	case desired != nil && current == nil:
+	case !haveService:
 		if err := r.client.Create(context.TODO(), desired); err != nil {
-			return nil, fmt.Errorf("failed to create dns service: %v", err)
+			return false, nil, fmt.Errorf("failed to create dns service: %v", err)
 		}
 		logrus.Infof("created dns service: %s/%s", desired.Namespace, desired.Name)
-	case desired != nil && current != nil:
-		if err := r.updateDNSService(current, desired); err != nil {
-			return nil, err
+		return r.currentDNSService(dns)
+	case haveService:
+		if updated, err := r.updateDNSService(current, desired); err != nil {
+			return true, current, err
+		} else if updated {
+			return r.currentDNSService(dns)
 		}
 	}
-	return r.currentDNSService(dns)
+	return true, current, nil
 }
 
-func (r *reconciler) currentDNSService(dns *operatorv1.DNS) (*corev1.Service, error) {
+func (r *reconciler) currentDNSService(dns *operatorv1.DNS) (bool, *corev1.Service, error) {
 	current := &corev1.Service{}
 	err := r.client.Get(context.TODO(), DNSServiceName(dns), current)
 	if err != nil {
 		if errors.IsNotFound(err) {
-			return nil, nil
+			return false, nil, nil
 		}
-		return nil, err
+		return false, nil, err
 	}
-	return current, nil
+	return true, current, nil
 }
 
 func desiredDNSService(dns *operatorv1.DNS, clusterIP string, daemonsetRef metav1.OwnerReference) *corev1.Service {
@@ -75,17 +78,17 @@ func desiredDNSService(dns *operatorv1.DNS, clusterIP string, daemonsetRef metav
 	return s
 }
 
-func (r *reconciler) updateDNSService(current, desired *corev1.Service) error {
+func (r *reconciler) updateDNSService(current, desired *corev1.Service) (bool, error) {
 	changed, updated := serviceChanged(current, desired)
 	if !changed {
-		return nil
+		return false, nil
 	}
 
 	if err := r.client.Update(context.TODO(), updated); err != nil {
-		return fmt.Errorf("failed to update dns service %s/%s: %v", updated.Namespace, updated.Name, err)
+		return false, fmt.Errorf("failed to update dns service %s/%s: %v", updated.Namespace, updated.Name, err)
 	}
 	logrus.Infof("updated dns service: %s/%s", updated.Namespace, updated.Name)
-	return nil
+	return true, nil
 }
 
 func serviceChanged(current, expected *corev1.Service) (bool, *corev1.Service) {
@@ -95,6 +98,8 @@ func serviceChanged(current, expected *corev1.Service) (bool, *corev1.Service) {
 		//
 		// TODO: Remove TopologyKeys when the service topology feature gate is enabled.
 		cmpopts.IgnoreFields(corev1.ServiceSpec{}, "ClusterIP", "TopologyKeys"),
+		cmp.Comparer(cmpServiceAffinity),
+		cmp.Comparer(cmpServiceType),
 		cmpopts.EquateEmpty(),
 	}
 
@@ -117,3 +122,23 @@ func serviceChanged(current, expected *corev1.Service) (bool, *corev1.Service) {
 
 	return true, updated
 }
+
+func cmpServiceAffinity(a, b corev1.ServiceAffinity) bool {
+	if len(a) == 0 {
+		a = corev1.ServiceAffinityNone
+	}
+	if len(b) == 0 {
+		b = corev1.ServiceAffinityNone
+	}
+	return a == b
+}
+
+func cmpServiceType(a, b corev1.ServiceType) bool {
+	if len(a) == 0 {
+		a = corev1.ServiceTypeClusterIP
+	}
+	if len(b) == 0 {
+		b = corev1.ServiceTypeClusterIP
+	}
+	return a == b
+}
